"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const fs = require("fs");
const chalk = require("chalk");
const cli_utils_1 = require("@ionic/cli-utils");
let DaemonCommand = class DaemonCommand extends cli_utils_1.Command {
    run(inputs, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            let { interval } = options;
            const updateInterval = Number(interval);
            const semver = this.env.load('semver');
            const config = yield this.env.config.load();
            if (!config.daemon) {
                this.env.log.info('Daemon is disabled.');
                return 1;
            }
            const f = yield this.env.daemon.getPid();
            if (f) {
                this.env.log.info(`Daemon pid file found: ${chalk.bold(this.env.daemon.pidFilePath)}`);
                const d = yield this.env.daemon.load();
                if (semver.gt(this.env.plugins.ionic.version, d.latestVersions.latest.ionic)) {
                    process.kill(Number(f));
                }
                else {
                    this.env.log.info('Daemon already running and up-to-date.');
                    return 0;
                }
            }
            this.env.log.info(`Writing ${chalk.bold(String(process.pid))} to daemon pid file (${chalk.bold(cli_utils_1.prettyPath(this.env.daemon.pidFilePath))}).`);
            yield this.env.daemon.setPid(process.pid);
            const updateFn = () => tslib_1.__awaiter(this, void 0, void 0, function* () {
                const config = yield this.env.config.load();
                if (!config.daemon) {
                    this.env.log.info('Daemon is disabled.');
                    process.exit();
                    return;
                }
                const d = yield this.env.daemon.load({ disk: true });
                const latestPlugins = Object.keys(d.latestVersions.latest);
                const latestUpdates = yield Promise.all(latestPlugins.map(pkg => cli_utils_1.pkgLatestVersion(this.env, pkg)));
                for (let i in latestPlugins) {
                    d.latestVersions.latest[latestPlugins[i]] = latestUpdates[i] || '';
                }
                if (typeof d.latestVersions.canary === 'object') {
                    const canaryPlugins = Object.keys(d.latestVersions.canary);
                    const canaryUpdates = yield Promise.all(canaryPlugins.map(pkg => cli_utils_1.pkgLatestVersion(this.env, pkg, 'canary')));
                    for (let i in canaryPlugins) {
                        d.latestVersions.canary[canaryPlugins[i]] = canaryUpdates[i] || '';
                    }
                }
                this.env.log.info('Writing daemon file.');
                yield this.env.daemon.save();
            });
            const cleanup = () => {
                try {
                    fs.unlinkSync(this.env.daemon.pidFilePath);
                }
                catch (e) {
                    if (e.code !== 'ENOENT') {
                        throw e;
                    }
                }
                process.exit();
            };
            process.on('exit', cleanup);
            process.on('SIGINT', cleanup);
            process.on('SIGTERM', cleanup);
            process.on('SIGHUP', cleanup);
            process.on('SIGBREAK', cleanup);
            yield updateFn();
            setInterval(updateFn, updateInterval * 1000);
        });
    }
};
DaemonCommand = tslib_1.__decorate([
    cli_utils_1.CommandMetadata({
        name: 'daemon',
        type: 'global',
        description: 'Ionic update checker daemon',
        options: [
            {
                name: 'interval',
                description: 'Interval, in seconds, to check for updates',
                default: '3600',
            },
        ],
        visible: false,
    })
], DaemonCommand);
exports.DaemonCommand = DaemonCommand;
